#include "OgreViewer.h"

Robot::Robot()
{
    alive = true;
    visible = true;
    vytvoren = false;

    poziceX = 0.0f;
    poziceY = 0.0f;
    rotaceKol = 0.0f;
    rotaceRobota = 0.0f;

    telo = NULL;
    kola = NULL;
    kolaNode = NULL;
    robotNode = NULL;
}

bool createRobot(OgreViewer *viewer, Robot *robot)
{
    robot->telo = viewer->scena->createEntity("telo.mesh");
    robot->kola = viewer->scena->createEntity("kola.mesh");

    robot->robotNode = viewer->scena->getRootSceneNode()->createChildSceneNode();
    robot->robotNode->attachObject(robot->telo);
    robot->kolaNode = robot->robotNode->createChildSceneNode();
    robot->kolaNode->attachObject(robot->kola);

    robot->vytvoren = true;
    //viewer->roboti[robot->id] = robot;
    //viewer->roboti.push_back(*robot);

    return true;
}

OgreViewer::OgreViewer()
{
    rychlostRotace = 90.0f;
    rychlostPosunu = 5.0f;
    nastaveno = false;

    // INICIALIZACE ROOT OBJEKTU
    std::cout << "Ogre : Vytvareni root objektu\n";
    Ogre::String logFileName = "log.txt";
    Ogre::String configFileName = "";
    Ogre::String pluginsFileName = "";

    Ogre::LogManager *lm = new Ogre::LogManager();
    lm->createLog(logFileName, true, false, false);

    root = new Ogre::Root(configFileName, pluginsFileName, "");

    // NACITANI PLUGINU
    std::cout << "Ogre : Nacitani pluginu\n";
    root->loadPlugin("RenderSystem_GL_d");
    //root->loadPlugin("RenderSystem_GL");

    // NACITANI ZDROJU
    std::cout << "Ogre : Nacitani zdroju\n";
    resManager = Ogre::ResourceGroupManager::getSingletonPtr();

    resManager->addResourceLocation("./Data/Textures", "FileSystem","General");
    resManager->addResourceLocation("./Data/Materials", "FileSystem", "General");
    resManager->addResourceLocation("./Data/Models", "FileSystem", "General");
    /*
    resManager->addResourceLocation("C:/Users/Martin/Desktop/OgreViewer/bin/Debug/Data/Textures","FileSystem","General");
    resManager->addResourceLocation("C:/Users/Martin/Desktop/OgreViewer/bin/Debug/Data/Materials","FileSystem","General");
    resManager->addResourceLocation("C:/Users/Martin/Desktop/OgreViewer/bin/Debug/Data/Models","FileSystem","General")
    */
    resManager->initialiseAllResourceGroups();

    // ZOBRAZENI KONFIGURACNIHO DIALOGU
    std::cout << "Ogre : Zobrazeni konfiguracniho dialogu\n";
    if (!root->showConfigDialog()){return;}

    // VYTVARENI OKNA
    std::cout << "Ogre : Vytvareni okna\n";
    root->initialise(true, "Moje vlastni okno");
    window = root->getAutoCreatedWindow();

    // INICIALIZACE OIS
    std::cout << "Ogre : Inicializace OIS\n";
    size_t handleOkna = 0;
    window->getCustomAttribute("WINDOW", &handleOkna);
    std::ostringstream handleOknaStream;
    handleOknaStream << handleOkna;
    OIS::ParamList seznamParametru;
    seznamParametru.insert(std::make_pair(std::string("WINDOW"), handleOknaStream.str()));

    inputManager = OIS::InputManager::createInputSystem(seznamParametru);
    klavesnice = static_cast<OIS::Keyboard*>(inputManager->createInputObject(OIS::OISKeyboard, false));
	mys = static_cast<OIS::Mouse*>(inputManager->createInputObject(OIS::OISMouse, false));

    // VYTVARENI TIMERU
    timer = root->getTimer();
    timer->reset();
    posledni_ms = timer->getMilliseconds();
    novy_ms = 0;

    // VYTVARENI MANAZERU SCENY
    std::cout << "Ogre : Vytvareni 3D Sceny\n";
    scena = root->createSceneManager(Ogre::ST_GENERIC, "3DScena");
    scena->setAmbientLight(Ogre::ColourValue(0.0, 0.0, 0.0));
    scena->setShadowTechnique(Ogre::SHADOWTYPE_STENCIL_ADDITIVE);

    // TVORBA A NASTAVENI KAMERY
    kamera = scena->createCamera("HlavniKamera");
    kamera->setNearClipDistance(1);
    kamera->setFarClipDistance(1000);
    kamera->setPosition(10, 10, 0);
    kamera->lookAt(0, 0, 0);
    kamera->setFixedYawAxis(true, Ogre::Vector3::UNIT_Z);
    kameraOuterNode = scena->getRootSceneNode()->createChildSceneNode();
    kameraNode = kameraOuterNode->createChildSceneNode();
    kameraNode->attachObject(kamera);

    // VYTVARENI SVETLA
    slunce = scena->createLight("Slunce");
    slunce->setPosition(10, 10, 10);
    slunce->setCastShadows(true);

    // VYTVARENI PODLAHY
    Ogre::Plane podlaha(Ogre::Vector3::UNIT_Y, -1);
    Ogre::MeshManager::getSingleton().createPlane("podlaha", "General", podlaha, 500, 500, 50, 50, true, 1, 5, 5, Ogre::Vector3::UNIT_Z);
    entitaPodlahy = scena->createEntity("mojePodlaha", "podlaha");
    entitaPodlahy->setCastShadows(false);
    entitaPodlahy->setMaterialName("materialPodlahy", "General");
    nodePodlahy = scena->getRootSceneNode()->createChildSceneNode();
    nodePodlahy->attachObject(entitaPodlahy);

/*
    // NACITANI MODELU
    telo = scena->createEntity("TeloModelu", "telo.mesh", "General");
    kola = scena->createEntity("KolaModelu", "kola.mesh", "General");

    modelNode = scena->getRootSceneNode()->createChildSceneNode();
    modelNode->attachObject(telo);
    kolaNode = modelNode->createChildSceneNode();
    kolaNode->attachObject(kola);
*/

    // VIEWPORT
    std::cout << "Ogre : Vytvareni viewportu\n";
    viewport = window->addViewport(kamera, 100, 0.0f, 0.0f, 1.0f, 1.0f);
    viewport->setBackgroundColour(Ogre::ColourValue(0.0f, 0.0f, 0.0f));

    //root->startRendering();

    nastaveno = true;
}

bool OgreViewer::renderFrame()
{
    //  UPDATE ROBOTU
    /*
    for (std::vector<Robot>::iterator odkaz = roboti.begin(); odkaz < roboti.end(); odkaz++)
    {
        Robot &temp = *odkaz;
        if (temp.alive)
        {
            temp.robotNode->setPosition(temp.poziceY, 0, temp.poziceX);     // NOT TESTED
            temp.robotNode->setOrientation(Ogre::Quaternion(Ogre::Radian(temp.rotaceRobota), Ogre::Vector3::UNIT_Y));
        }
        else if (temp.visible)
        {
            // ODSTRANENI MRTVEHO ROBOTA
            scena->getRootSceneNode()->removeChild(temp.robotNode);     // NOT TESTED
            temp.visible = false;                                       // NOT TESTED
        }
    }*/

    for (std::map<int, Robot>::iterator it = roboti.begin(); it != roboti.end(); it++)
    {
        int id = it->first;
        //if (!it->second->vytvoren){createRobot(this, it);}
        Robot &temp = (*it).second;
        if (!temp.vytvoren){createRobot(this, &temp);}
        if (temp.alive)
        {
            temp.robotNode->setPosition(-temp.poziceY, 0, temp.poziceX);     // NOT TESTED
            temp.robotNode->setOrientation(Ogre::Quaternion(Ogre::Radian(-1 * (temp.rotaceRobota - 1.57f)), Ogre::Vector3::UNIT_Y));
            temp.kolaNode->roll(Ogre::Radian(temp.rotaceKol));
        }
        else if (temp.visible)
        {
            // ODSTRANENI MRTVEHO ROBOTA
            scena->getRootSceneNode()->removeChild(temp.robotNode);     // NOT TESTED
            temp.visible = false;                                       // NOT TESTED
        }
    }

    // ZJISTENI CASU OD POSLEDNIHO SNIMKU
    float rozdil_s = 0.0f;
    novy_ms = timer->getMilliseconds();
    unsigned long rozdil_ms = novy_ms - posledni_ms;
    posledni_ms = novy_ms;
    rozdil_s = float(rozdil_ms) / 1000.0f;

    // ZACHYCENI ZPRAV KLAVESNICE A MYSI
    klavesnice->capture();

    if (klavesnice->isKeyDown(OIS::KC_ESCAPE))
    {
        return false;
    }

    mys->capture();
    const OIS::MouseState &stavMysi = mys->getMouseState();

    float posunMysiX = float(stavMysi.X.rel) / float(window->getWidth());
    float posunMysiY = float(stavMysi.Y.rel) / float(window->getHeight());

    if (stavMysi.buttonDown(OIS::MB_Right))      // pohyb kamery
    {
        kameraOuterNode->translate(0, 0, -1 * posunMysiX * rychlostPosunu, Ogre::Node::TS_LOCAL);
        kameraOuterNode->translate(posunMysiY * rychlostPosunu, 0, 0, Ogre::Node::TS_LOCAL);
    }
    else                               // pohled treti osoby
    {
        kameraOuterNode->yaw(Ogre::Degree(-1 * posunMysiX * rychlostRotace), Ogre::Node::TS_PARENT);   // rotace okolo osy Y
        kameraNode->rotate(Ogre::Quaternion(Ogre::Degree(posunMysiY*rychlostRotace), Ogre::Vector3::UNIT_Z)); // rotace okolo osy X
    }

    // VYRENDROVANI JEDNOHO SNIMKU
    root->renderOneFrame();

    // ODEVZDANI ZPRAV SYSTEMU
    Ogre::WindowEventUtilities::messagePump();

    return true;
}
