#include "OgreViewer.h"

Robot::Robot(int ID)
{
    id = ID;
    alive = true;
    visible = true;

    poziceX = 0.0f;
    poziceY = 0.0f;
    rotaceKol = 0.0f;
    rotaceRobota = 0.0f;

    telo = NULL;
    kola = NULL;
    kolaNode = NULL;
    robotNode = NULL;
}

bool createRobot(OgreViewer *viewer, Robot *robot)
{
    robot->telo = viewer->scena->createEntity("telo.mesh");
    robot->kola = viewer->scena->createEntity("kola.mesh");

    robot->robotNode = viewer->scena->getRootSceneNode()->createChildSceneNode();
    robot->robotNode->attachObject(robot->telo);
    robot->kolaNode = robot->robotNode->createChildSceneNode();
    robot->kolaNode->attachObject(robot->kola);

    viewer->roboti.push_back(*robot);

    return true;
}

OgreViewer::OgreViewer()
{
    rychlostRotace = 5.0f;
    rychlostPosunu = 1.2f;

    // ROOT INITIALIZATION
    Ogre::String logFileName = "log.txt";
    Ogre::String configFileName = "";
    Ogre::String pluginsFileName = "";
    root = new Ogre::Root(configFileName, pluginsFileName, logFileName);

    // LOADING PLUGINS
    root->loadPlugin("RenderSystem_GL_d");
    //root->loadPlugin("RenderSystem_GL");

    // LOADING RESOURCES
    resManager = Ogre::ResourceGroupManager::getSingletonPtr();

    /*
    resManager->addResourceLocation("./Data/Textures", "FileSystem","General");
    resManager->addResourceLocation("./Data/Materials", "FileSystem", "General");
    resManager->addResourceLocation("./Data/Models", "FileSystem", "General");
    */
    resManager->addResourceLocation("C:/Users/Martin/Desktop/OgreViewer/bin/Debug/Data/Textures","FileSystem","General");
    resManager->addResourceLocation("C:/Users/Martin/Desktop/OgreViewer/bin/Debug/Data/Materials","FileSystem","General");
    resManager->addResourceLocation("C:/Users/Martin/Desktop/OgreViewer/bin/Debug/Data/Models","FileSystem","General");
    resManager->initialiseAllResourceGroups();

    // OGRE CONFIGURATION
    if (!root->showConfigDialog()){return;}

    // CREATING WINDOW
    root->initialise(true, "Moje vlastni okno");
    window = root->getAutoCreatedWindow();

    // CREATING SCENE MANAGER
    scena = root->createSceneManager(Ogre::ST_GENERIC, "3DScena");
    scena->setAmbientLight(Ogre::ColourValue(0.0, 0.0, 0.0));
    scena->setShadowTechnique(Ogre::SHADOWTYPE_STENCIL_ADDITIVE);

    // CREATING CAMERA
    kamera = scena->createCamera("HlavniKamera");
    kamera->setNearClipDistance(1);
    kamera->setFarClipDistance(1000);
    kamera->setPosition(10, 10, 0);
    kamera->lookAt(0, 0, 0);
    kamera->setFixedYawAxis(true, Ogre::Vector3::UNIT_Z);
    kameraOuterNode = scena->getRootSceneNode()->createChildSceneNode();
    kameraNode = kameraOuterNode->createChildSceneNode();
    kameraNode->attachObject(kamera);

    // CREATING LIGHT
    slunce = scena->createLight("Slunce");
    slunce->setPosition(10, 10, 10);
    slunce->setCastShadows(true);

    // CREATING FLOORING
    Ogre::Plane podlaha(Ogre::Vector3::UNIT_Y, -1);
    Ogre::MeshManager::getSingleton().createPlane("podlaha", "General", podlaha, 500, 500, 50, 50, true, 1, 5, 5, Ogre::Vector3::UNIT_Z);
    entitaPodlahy = scena->createEntity("mojePodlaha", "podlaha");
    entitaPodlahy->setCastShadows(false);
    entitaPodlahy->setMaterialName("materialPodlahy", "General");
    nodePodlahy = scena->getRootSceneNode()->createChildSceneNode();
    nodePodlahy->attachObject(entitaPodlahy);

/*
    // CREATING MODEL
    telo = scena->createEntity("TeloModelu", "telo.mesh", "General");
    kola = scena->createEntity("KolaModelu", "kola.mesh", "General");

    modelNode = scena->getRootSceneNode()->createChildSceneNode();
    modelNode->attachObject(telo);
    kolaNode = modelNode->createChildSceneNode();
    kolaNode->attachObject(kola);
*/

    // VIEWPORT
    viewport = window->addViewport(kamera, 100, 0.0f, 0.0f, 1.0f, 1.0f);
    viewport->setBackgroundColour(Ogre::ColourValue(0.0f, 0.0f, 0.0f));

    //root->startRendering();
}

bool OgreViewer::renderFrame()
{
    //  UPDATE ROBOTU
    for (vector<Robot>::iterator odkaz = roboti.begin(); odkaz < roboti.end(); odkaz++)
    {
        Robot &temp = *odkaz;
        if (temp.alive)
        {
            temp.robotNode->setPosition(temp.poziceY, 0, temp.poziceX);     // NOT TESTED
            temp.robotNode->setOrientation(Ogre::Quaternion(Ogre::Radian(temp.rotaceRobota), Ogre::Vector3::UNIT_Y));
        }
        else if (temp.visible)
        {
            // ODSTRANENI MRTVEHO ROBOTA
            scena->getRootSceneNode()->removeChild(temp.robotNode);     // NOT TESTED
            temp.visible = false;                                       // NOT TESTED
        }
    }

    root->renderOneFrame();

    return true;
}
