#include <iostream>
#include <memory>

#include "Ogre.h"
#include "OIS.h"

using namespace std;

Ogre::String hvezdicky = "\n******************************************\n\n";
Ogre::String logFileName = "ogre.log";

auto_ptr<Ogre::Root> root (new Ogre::Root("", "", logFileName));
//Ogre::Root *root = new Ogre::Root("", "", logFileName);

int main()
{
	// Inicializace zdrojù
	cout << hvezdicky << "Nacitani zdroju\n" << hvezdicky;

    // vytvoření správce zdrojů a přidání složek
	Ogre::ResourceGroupManager *resManager = Ogre::ResourceGroupManager::getSingletonPtr();
	resManager->addResourceLocation("./Data/Materials", "FileSystem", "General");
	resManager->addResourceLocation("./Data/Models", "FileSystem", "General");

	Ogre::ResourceGroupManager::getSingletonPtr()->initialiseAllResourceGroups();


	// Inicializace programu
	cout << hvezdicky << "Nacitani dynamickych knihoven\n" << hvezdicky;

    // načítání pluginů (v debug módu musí obsahovat příponu _d)
	root->loadPlugin("RenderSystem_GL");
	root->loadPlugin("RenderSystem_Direct3D9");

	cout << hvezdicky << "Konfigurace OGRE\n" << hvezdicky;

    // zobrazení konfiguračního dialogu (pokud uživatel klikne na cancel, program se ukončí)
	bool rtn = root->showConfigDialog();
	if (rtn == false){return 0;}

	cout << hvezdicky << "Tvorba okna\n" << hvezdicky;

    // vytvoření okna podle vybraných parametrů
	root->initialise(true, "Moje Okno");

	// Inicializace scény
	cout << hvezdicky << "Vytvareni 3D sceny\n" << hvezdicky;

	Ogre::SceneManager *scena = root->createSceneManager(Ogre::ST_GENERIC, "3DScena");
	scena->setAmbientLight(Ogre::ColourValue(0.0, 0.0, 0.0));
	scena->setShadowTechnique(Ogre::SHADOWTYPE_STENCIL_ADDITIVE);

    /*
    // zobrazení 3D os (X, Y, Z)
    Ogre::SceneNode *axisNode = scena->getRootSceneNode()->createChildSceneNode();

    Ogre::ManualObject *osaX = new Ogre::ManualObject("osaX");
    osaX->begin("materialPlochy", Ogre::RenderOperation::OT_LINE_LIST);
    osaX->position(-10, 0, 0);
    osaX->position(10, 0, 0);
    osaX->end();

    Ogre::ManualObject *osaY = new Ogre::ManualObject("osaY");
    osaY->begin("materialPlochy", Ogre::RenderOperation::OT_LINE_LIST);
    osaY->position(0, -10, 0);
    osaY->position(0, 10, 0);
    osaY->end();

    Ogre::ManualObject *osaZ = new Ogre::ManualObject("osaZ");
    osaZ->begin("materialPlochy", Ogre::RenderOperation::OT_LINE_LIST);
    osaZ->position(0, 0, -10);
    osaZ->position(0, 0, 10);
    osaZ->end();

    axisNode->attachObject(osaX);
    axisNode->attachObject(osaY);
    axisNode->attachObject(osaZ);
    */

    // plocha
    Ogre::SceneNode *plochaNode = scena->getRootSceneNode()->createChildSceneNode();

    Ogre::ManualObject *plocha = new Ogre::ManualObject("Plocha");
    plocha->begin("materialPlochy", Ogre::RenderOperation::OT_TRIANGLE_STRIP);
    plocha->position(100, 0, 100);
    plocha->position(-100, 0, 100);
    plocha->position(100, 0, -100);
    plocha->position(-100, 0, -100);
    plocha->end();

    plochaNode->attachObject(plocha);
    plochaNode->rotate(Ogre::Quaternion(Ogre::Degree(180), Ogre::Vector3::UNIT_X));

    // tvorba a nastavení kamery
	Ogre::Camera *kamera = scena->createCamera("HlavniKamera");
	kamera->setNearClipDistance(1);
	kamera->setFarClipDistance(1000);
	kamera->setPosition(10, 10, 0);
	kamera->lookAt(0, 1, 0);
	kamera->setFixedYawAxis(true, Ogre::Vector3::UNIT_Z);

    // přiřazení kamery do uzlů (dva jsou pro možnost pohybu kamery částečně nezávisle na rotaci kamery)
	Ogre::SceneNode *kameraOuterNode = scena->getRootSceneNode()->createChildSceneNode();
	Ogre::SceneNode *kameraNode  = kameraOuterNode->createChildSceneNode();
	kameraNode->attachObject(kamera);

    // vytvoření světla
	Ogre::Light *svetlo = scena->createLight("Slunce");
	svetlo->setPosition(10, 10, 10);
	svetlo->setCastShadows(true);

    // vytvoření entit = vlastních objektů
	Ogre::Entity *telo = scena->createEntity("TeloModelu", "telo.mesh", "General");
	Ogre::Entity *kola = scena->createEntity("Kola", "kola.mesh", "General");

    // vytvoření uzlu (modelNode) a přiřazení entity a druhého uzlu (kola mají vlastní uzel aby se mohla otáčet)
	Ogre::SceneNode *modelNode = scena->getRootSceneNode()->createChildSceneNode();
	modelNode->attachObject(telo);
	Ogre::SceneNode *kolaNode = modelNode->createChildSceneNode();
	kolaNode->attachObject(kola);

    // nastavení správní orientace a polohy modelu (exporter z Blenderu je už poněkud starší a trošku blbne)
	modelNode->rotate(Ogre::Quaternion(Ogre::Degree(-90), Ogre::Vector3::UNIT_Z));
	modelNode->translate(Ogre::Vector3(0, 1, 0));


    // vytvoření viewportu (=spojení mezi výstupem renderu a určitou částí okna)
    Ogre::RenderWindow *window = root->getAutoCreatedWindow();
	Ogre::Viewport *viewport = window->addViewport(kamera, 100, 0.0f, 0.0f, 1.0f, 1.0f);
	viewport->setBackgroundColour(Ogre::ColourValue(0.0f, 0.0f, 0.0f));

	// inicializace OIS (Object-oriented Input System)
	cout << hvezdicky << "Inicializace OIS\n" << hvezdicky;

    // je potřba získat handle okna ve formátu std::string
	size_t handleOkna = 0;
	window->getCustomAttribute("WINDOW", &handleOkna);
	ostringstream handleOknaStream;
	handleOknaStream << handleOkna;

    // handle se pak přidá do seznamu parametrù
	OIS::ParamList seznamParametru;
	seznamParametru.insert(make_pair(string("WINDOW"), handleOknaStream.str()));

    // a vytvoří se vstupní systém
	OIS::InputManager *inputManager = OIS::InputManager::createInputSystem(seznamParametru);

    // který pak můžeme použít k vytvoření objektů zastupujících klávesnici a myš
	OIS::Keyboard *klavesnice = static_cast<OIS::Keyboard*>(inputManager->createInputObject(OIS::OISKeyboard, false));
	OIS::Mouse *mys = static_cast<OIS::Mouse*>(inputManager->createInputObject(OIS::OISMouse, false));

    // pár proměnných pro práci s časem
	Ogre::Timer *timer = root->getTimer();
	timer->reset();
	unsigned long posledni_ms = timer->getMilliseconds();
	unsigned long novy_ms = 0;
	float rozdil_s = 0.0f;

	// hlavní smyčka programu
	cout << hvezdicky << "Rendering\n" << hvezdicky;

	while(!window->isClosed())
	{
		// získání doby od posledního snímku
		novy_ms = timer->getMilliseconds();
		unsigned long rozdil_ms = novy_ms - posledni_ms;
		posledni_ms = novy_ms;
		rozdil_s = float(rozdil_ms) / 1000.0f;

		// zpracování vstupu z klávesnice
		klavesnice->capture();
		Ogre::Vector3 posun(Ogre::Vector3::ZERO);
		float rychlost = 3.0f;

		if((klavesnice->isKeyDown(OIS::KC_LEFT)) || (klavesnice->isKeyDown(OIS::KC_A)))
		{
			modelNode->rotate(Ogre::Quaternion(Ogre::Radian(rozdil_s * rychlost), Ogre::Vector3::UNIT_Y), Ogre::Node::TS_WORLD);
		}
		if((klavesnice->isKeyDown(OIS::KC_RIGHT)) || (klavesnice->isKeyDown(OIS::KC_D)))
		{
			modelNode->rotate(Ogre::Quaternion(Ogre::Radian(-1 * rozdil_s * rychlost), Ogre::Vector3::UNIT_Y), Ogre::Node::TS_WORLD);
		}
		if((klavesnice->isKeyDown(OIS::KC_UP)) || (klavesnice->isKeyDown(OIS::KC_W)))
		{
			posun.z = rozdil_s * rychlost;
			kolaNode->rotate(Ogre::Quaternion(Ogre::Radian(rozdil_s * rychlost * 0.7f), Ogre::Vector3::UNIT_Y), Ogre::Node::TS_LOCAL);
		}
		if((klavesnice->isKeyDown(OIS::KC_DOWN)) || (klavesnice->isKeyDown(OIS::KC_S)))
		{
			posun.z = - (rozdil_s * rychlost);
			kolaNode->rotate(Ogre::Quaternion(Ogre::Radian(-1 * rozdil_s * rychlost * 0.7f), Ogre::Vector3::UNIT_Y), Ogre::Node::TS_LOCAL);
		}

		if (klavesnice->isKeyDown(OIS::KC_SPACE))   // reset polohy modelu
		{
			modelNode->setPosition(Ogre::Vector3(0, 1, 0));
		}

		if (klavesnice->isKeyDown(OIS::KC_ESCAPE))
		{
			break;
		}

		modelNode->translate(posun, Ogre::Node::TS_LOCAL);

		//zpracování vstupu z myši

		mys->capture();
		const OIS::MouseState &stavMysi = mys->getMouseState();

        // získání relativního posunu myši nezávisle na rozlišení okna
		float posunMysiX = float(stavMysi.X.rel) / float (window->getWidth());
		float posunMysiY = float(stavMysi.Y.rel) / float (window->getHeight());

		float rychlostRotace = 90.0f;
		float rychlostPohybu = 20.0f;

        if (stavMysi.buttonDown(OIS::MB_Right))      // pohyb kamery
        {
            kameraOuterNode->translate(0, 0, -1 * posunMysiX * rychlostPohybu, Ogre::Node::TS_LOCAL);
            kameraOuterNode->translate(posunMysiY * rychlostPohybu, 0, 0, Ogre::Node::TS_LOCAL);
        }
        else                                        // pohled třetí osoby
        {
            kameraOuterNode->yaw(Ogre::Degree(posunMysiX * rychlostRotace), Ogre::Node::TS_PARENT);   // rotace okolo osy Y
            kameraNode->rotate(Ogre::Quaternion(Ogre::Degree(posunMysiY*rychlostRotace), Ogre::Vector3::UNIT_Z)); // rotace okolo osy X
        }

		// rendering snímku
		root->renderOneFrame();

		// zpracování zpráv systémem ( = posuv okna po obrazovce, ...)
		Ogre::WindowEventUtilities::messagePump();
	}

	cout << hvezdicky << "Program byl ukoncen" << hvezdicky;
	return 0;
}
